@@ -236,6 +236,11 @@ pub enum Action {
     /// Print the config being in use, default config if none.
     #[serde(alias = "config")]
     Config,
+
+    /// List all files in depth first sorted order in which they
+    /// would be checked.
+    #[serde(alias = "files")]
+    ListFiles,
 }
 
 impl Action {
@@ -321,11 +326,17 @@ impl Action {
     /// Purpose was to check, checking complete, so print the results.
     fn check(&self, suggestions_per_path: SuggestionSet, _config: &Config) -> Result<Finish> {
         let mut count = 0usize;
-        for (_path, suggestions) in suggestions_per_path {
-            count += suggestions.len();
+        for (path, suggestions) in suggestions_per_path {
+            let n = suggestions.len();
+            count += n;
             for suggestion in suggestions {
                 println!("{}", suggestion);
             }
+            if n == 0 {
+                info!("✅ {}", path.as_path().display());
+            } else {
+                info!("❌ {} : {}", path.as_path().display(), n);
+            }
         }
         Ok(Finish::MistakeCount(count))
     }
@@ -365,7 +376,7 @@ impl Action {
                     Ok(Finish::MistakeCount(n))
                 }
             }
-            Self::Config | Self::Version | Self::Help => {
+            Self::Config | Self::Version | Self::Help | Self::ListFiles => {
                 unreachable!("Should have been handled way earlier")
             }
         }
