@@ -98,26 +98,39 @@ impl Config {
         Ok(toml::from_str(s.as_ref())?)
     }
 
-    pub fn load_from<P: AsRef<Path>>(path: P) -> Result<Self> {
-        let mut file = fs::File::open(path.as_ref().to_str().unwrap())?;
-        let mut contents = String::with_capacity(1024);
-        file.read_to_string(&mut contents)?;
+    pub fn load_from<P: AsRef<Path>>(path: P) -> Result<Option<Self>> {
+        let (contents, path) = match Self::load_content(path) {
+            Err(e) if e.kind() == std::io::ErrorKind::NotFound => {
+                return Ok(None);
+            }
+            Err(e) => bail!(e),
+            Ok(contents) => contents,
+        };
         Self::parse(&contents)
             .wrap_err_with(|| {
                 eyre!(
                     "Syntax of a given config file({}) is broken",
-                    path.as_ref().display()
+                    path.display()
                 )
             })
             .and_then(|mut cfg| {
-                if let Some(base) = path.as_ref().parent() {
+                if let Some(base) = path.parent() {
                     cfg.sanitize_paths(base)?;
                 }
-                Ok(cfg)
+                Ok(Some(cfg))
             })
     }
 
-    pub fn load() -> Result<Self> {
+    pub fn load_content<P: AsRef<Path>>(path: P) -> std::io::Result<(String, PathBuf)> {
+        let path = path.as_ref().canonicalize()?;
+        let mut file = fs::File::open(&path)?;
+
+        let mut contents = String::with_capacity(1024);
+        file.read_to_string(&mut contents)?;
+        Ok((contents, path))
+    }
+
+    pub fn load() -> Result<Option<Self>> {
         if let Some(base) = directories::BaseDirs::new() {
             Self::load_from(
                 base.config_dir()
